{***************************************************************
 *
 * Unit Name : FormMisc
 * Purpose   : Form Miscellaneous
 * Author    : Eugen Mihailescu
 * Company   : EM Quicksoft Romania SRL
 * Copyright : 1998,2002  All rights reserved.
 * Web Site  : http://www.em-quicksoft.com
 * Support   : support@em-quicksoft.com
 * History   : 03/19/2000
 *
 * Disclaimer: The objectives of this program is only educational.
 *             You MUST NOT use it without getting any material benefit on it
 *             You have the right to share this code with others without removing
 *             the copyright and/or disclaimer notice.
 *             You are allowed to include its code in your application
 *             only if it was designed for non commercial purpose,
 *             otherwise we claim $10 for each copy of your
 *             application which contains this code inside.
 ****************************************************************}

Unit FormMisc;

Interface

Uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  Shellapi;

Type
  TFooClass = Class(TControl);
  TFormMisc = Class(TComponent)
  private
    fFileName: TFileName;
    ScreenWidth, ScreenHeight, NewFormWidth, OldFormWidth: Word;
    fCompanyName: String;
    fFileDescription: String;
    fFileVersion: String;
    fInternalName: String;
    fLegalCopyright: String;
    fLegalTradeMark: String;
    fOriginalFileName: String;
    fProductName: String;
    fProductVersion: String;
    fComments: String;
    fApplicationEXEName: String;
    fOwner: TComponent;
    fShowIconOnTaskBar: Boolean;
    fTaskBarHint: String;
    Procedure CreateIconTaskBar;
    Procedure DestroyIconTaskbar;
    Procedure SetShowIconOnTaskBar(Value: Boolean);
  protected
    Procedure SetVersion(Value: TFileName);
  public
    Procedure IconCallBackMessage(Var Mess: TMessage); message WM_USER + 100;
    Constructor Create(Owner: TComponent); override;
    Destructor Destroy; override;
    Procedure CheckFormSize(Owner: TComponent);
    Procedure ReadVersion;
  published
    Property CompanyName: String read fCompanyName write fCompanyName;
    Property FileDescription: String read fFileDescription write fFileDescription;
    Property FileVersion: String read fFileVersion write fFileVersion;
    Property InternalName: String read fInternalName write fInternalName;
    Property LegalCopyright: String read fLegalCopyright write fLegalCopyright;
    Property LegalTradeMark: String read fLegalTradeMark write fLegalTradeMark;
    Property OriginalFileName: String read fOriginalFileName write fOriginalFileName;
    Property ProductName: String read fProductName write fProductName;
    Property ProductVersion: String read fProductVersion write fProductVersion;
    Property Comments: String read fComments write fComments;
    Property ApplicationEXEName: String read fApplicationEXEName write fApplicationEXEName;
    Property TaskBarHint: String read fTaskBarHint write fTaskBarHint;
    Property ShowIconOnTaskBar: Boolean read fShowIconOnTaskBar write SetShowIconOnTaskBar;
    Property FileName: TFileName read fFileName write SetVersion;
  End;

Procedure Register;

Implementation
//******************* TFormMisc.Destroy *************************

Destructor TFormMisc.Destroy;
Begin
  Inherited Destroy;
End;
//******************* TFormMisc.Create *************************

Constructor TFormMisc.Create(Owner: TComponent);
Begin
  Inherited Create(Owner);              // Initialize inherited parts
  fOwner := Owner;
  CheckFormSize(Owner);
  fFileName := Application.ExeName;
  ReadVersion;
  fTaskBarHint := fFileDescription;
  If fShowIconOnTaskBar Then CreateIconTaskBar;
End;
//******************* TFormMisc.CheckFormSize *************************

Procedure TFormMisc.CheckFormSize(Owner: TComponent);
Var
  I                 : Integer;
Begin
  {  with (Owner as TForm) do
    begin
      OldFormWidth := Width;
      ScreenHeight := Screen.Height;
      ScreenWidth := Screen.Width;
      scaled := true;
      if (screen.width <> OldFormWidth) then
      begin
        height := longint(height) * longint(screen.height) div ScreenHeight;
        width := longint(width) * longint(screen.width) div ScreenWidth;
        scaleBy(screen.width, ScreenWidth);

        for i := ControlCount - 1 downto 0 do
          TFooClass(Controls[i]).Font.Size :=
            (NewFormWidth div OldFormWidth) *
            TFooClass(Controls[i]).Font.Size;
      end;
      NewFormWidth := Width;
    end;}
End;
//******************* TFormMisc.ReadVersion *************************

Procedure TFormMisc.ReadVersion;
Const
  InfoNum           = 10;
  InfoStr           : Array[1..InfoNum] Of String =
    ('CompanyName', 'FileDescription', 'FileVersion', 'InternalName',
    'LegalCopyright', 'LegalTradeMarks', 'OriginalFilename',
    'ProductName', 'ProductVersion', 'Comments');
Var
  S                 : String;
  Len, I            : Cardinal;
  n                 : Cardinal;
  Buf               : PChar;
  Value             : PChar;
Begin
  S := fFileName;
  fApplicationEXEName := S;
  n := GetFileVersionInfoSize(PChar(S), n);
  If n > 0 Then
    Begin
      Buf := AllocMem(n);
      //    Memo1.Lines.Add('FileVersionInfoSize=' + IntToStr(n));
      GetFileVersionInfo(PChar(S), 0, n, Buf);
      If VerQueryValue(Buf, PChar('StringFileInfo\040904E4\' +
        InfoStr[1]), Pointer(Value), Len) Then
        fCompanyName := Value
      Else
        fCompanyName := '';
      If VerQueryValue(Buf, PChar('StringFileInfo\040904E4\' +
        InfoStr[2]), Pointer(Value), Len) Then
        fFileDescription := Value
      Else
        fFileDescription := '';
      If VerQueryValue(Buf, PChar('StringFileInfo\040904E4\' +
        InfoStr[3]), Pointer(Value), Len) Then
        fFileVersion := Value
      Else
        fFileVersion := '';
      If VerQueryValue(Buf, PChar('StringFileInfo\040904E4\' +
        InfoStr[4]), Pointer(Value), Len) Then
        fInternalName := Value
      Else
        fInternalName := '';
      If VerQueryValue(Buf, PChar('StringFileInfo\040904E4\' +
        InfoStr[5]), Pointer(Value), Len) Then
        fLegalCopyright := Value
      Else
        fLegalCopyright := '';
      If VerQueryValue(Buf, PChar('StringFileInfo\040904E4\' +
        InfoStr[6]), Pointer(Value), Len) Then
        fLegalTradeMark := Value
      Else
        fLegalTradeMark := '';
      If VerQueryValue(Buf, PChar('StringFileInfo\040904E4\' +
        InfoStr[7]), Pointer(Value), Len) Then
        fOriginalFileName := Value
      Else
        fOriginalFileName := '';
      If VerQueryValue(Buf, PChar('StringFileInfo\040904E4\' +
        InfoStr[8]), Pointer(Value), Len) Then
        fProductName := Value
      Else
        fProductName := '';
      If VerQueryValue(Buf, PChar('StringFileInfo\040904E4\' +
        InfoStr[9]), Pointer(Value), Len) Then
        fProductVersion := Value
      Else
        fProductVersion := '';
      If VerQueryValue(Buf, PChar('StringFileInfo\040904E4\' +
        InfoStr[10]), Pointer(Value), Len) Then
        fComments := Value
      Else
        fComments := '';
      FreeMem(Buf, n);
    End
  Else
    Begin
      fCompanyName := '';
      fFileDescription := '';
      fFileVersion := '';
      fInternalName := '';
      fLegalCopyright := '';
      fLegalTradeMark := '';
      fOriginalFileName := '';
      fProductName := '';
      fProductVersion := '';
      fComments := '';
    End;
End;
//******************* TFormMisc.CreateIconTaskBar *************************

Procedure TFormMisc.CreateIconTaskBar;
Var
  nid               : TNotifyIconData;
Begin
  With nid Do
    Begin
      cbSize := SizeOf(TNotifyIconData);
      Wnd := (Owner As TForm).Handle;
      uID := 1;
      uFlags := NIF_ICON Or NIF_MESSAGE Or NIF_TIP;
      uCallbackMessage := WM_USER + 100;
      hIcon := Application.Icon.Handle;
      StrCopy(szTip, PChar(fTaskBarHint));
    End;
  Shell_NotifyIcon(NIM_ADD, @nid);
End;
//******************* TFormMisc.DestroyIconTaskbar *************************

Procedure TFormMisc.DestroyIconTaskbar;
Var
  nid               : TNotifyIconData;
Begin
  With nid Do
    Begin
      cbSize := SizeOf(TNotifyIconData);
      Wnd := (fOwner As TForm).Handle;
      uID := 1;
      uFlags := NIF_ICON Or NIF_MESSAGE Or NIF_TIP;
      uCallbackMessage := WM_USER + 100;
      hIcon := Application.Icon.Handle;
      StrCopy(szTip, PChar(fTaskBarHint));
      // All the above is probably not needed.
    End;
  Shell_NotifyIcon(NIM_DELETE, @nid);
End;
//******************* TFormMisc.SetShowIconOnTaskBar *************************

Procedure TFormMisc.SetShowIconOnTaskBar(Value: Boolean);
Begin
  If Value Then
    CreateIconTaskBar
  Else
    DestroyIconTaskBar;
  fShowIconOnTaskBar := Value;
End;
//******************* TFormMisc.IconCallBackMessage *************************

Procedure TFormMisc.IconCallBackMessage(Var Mess: TMessage);
Var
  sEventLog         : String;
Begin
  Case Mess.lParam Of
    // Do whatever you wish here. For example popup up a menu on a right click.
    WM_LBUTTONDBLCLK: sEventLog := 'Left Double Click';
    WM_LBUTTONDOWN: sEventLog := 'Left Down';
    WM_LBUTTONUP: sEventLog := 'Left Up';
    WM_MBUTTONDBLCLK: sEventLog := 'M Dbl';
    WM_MBUTTONDOWN: sEventLog := 'M D';
    WM_MBUTTONUP: sEventLog := 'M U';
    WM_MOUSEMOVE: sEventLog := 'movement';
    WM_MOUSEWHEEL: sEventLog := 'Wheel';
    WM_RBUTTONDBLCLK: sEventLog := 'r dbl';
    WM_RBUTTONDOWN: sEventLog := 'r down';
    WM_RBUTTONUP: sEventLog := 'r up';
  End;
  ShowMessage(seventlog);
End;
//******************* TFormMisc.SetVersion *************************

Procedure TFormMisc.SetVersion(Value: TFileName);
Begin
  fFileName := Value;
  ReadVersion;
End;

Procedure Register;
Begin
  RegisterComponents('Eugen''s Components', [TFormMisc]);
End;

End.

